/*****************************************************************************
 *	Includes / define
 ****************************************************************************/
//system includes
#include	<math.h>
#include	<stdio.h>
#include	<stdlib.h>

//other includes
#include	"dll.h"

//constants
#define		PI	((double)3.14159265358979323846264338327950288419716939937510)

/*****************************************************************************
 *	Definition of main functions
 ****************************************************************************/

extern "C"
{
__declspec(dllexport) int Translate(int dim, int dim_x, int dim_y, double* a, double* shifts, double* b)
{
	double x0, y0, dx, dy;
	int x1, x2, y1, y2;

    for (int p=0; p<dim; p++)
    {
        /* visit all pixels of the output image and assign their value */
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            x0 = (double)i - shifts[0];
            y0 = (double)j - shifts[1];

            x1 = int(floor(x0));
            x2 = x1+1;
            y1 = int(floor(y0));
            y2 = y1+1;
            dx = x0-floor(x0);
            dy = y0-floor(y0);

            b[p*dim_x*dim_y + i*dim_y + j] = 0.0;

            if ((x1 >= 0) && (dim_x - 1 >= x1) && (y1 >= 0) && (dim_y - 1 >= y1))
                b[p*dim_x*dim_y + i*dim_y + j] += (1.0-dx)*(1.0-dy)*a[p*dim_x*dim_y + x1*dim_y + y1];
            if ((x2 >= 0) && (dim_x - 1 >= x2) && (y1 >= 0) && (dim_y - 1 >= y1))
                b[p*dim_x*dim_y + i*dim_y + j] += dx*(1.0-dy)*a[p*dim_x*dim_y + x2*dim_y + y1];
            if ((x1 >= 0) && (dim_x - 1 >= x1) && (y2 >= 0) && (dim_y - 1 >= y2))
                b[p*dim_x*dim_y + i*dim_y + j] += (1.0-dx)*dy*a[p*dim_x*dim_y + x1*dim_y + y2];
            if ((x2 >= 0) && (dim_x - 1 >= x2) && (y2 >= 0) && (dim_y - 1 >= y2))
                b[p*dim_x*dim_y + i*dim_y + j] += dx*dy*a[p*dim_x*dim_y + x2*dim_y + y2];
        }}
    }
	return(0);
}
__declspec(dllexport) int Rotate(int dim, int dim_x, int dim_y, double* a, double* axis, double angle, double* b)
{
	double x0, y0, dx, dy;
	int x1, x2, y1, y2;

    /* conversion from degrees to radians */
    //positive angles correspond to clockwise rotations
    angle = (PI/180.0) * angle;

    for (int p=0; p<dim; p++)
    {
        /* visit all pixels of the output image and assign their value */
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

            x0 = axis[0] + cos(angle)*((double)i - axis[0]) + sin(angle)*((double)j - axis[1]);
            y0 = axis[1] - sin(angle)*((double)i - axis[0]) + cos(angle)*((double)j - axis[1]);

            x1 = int(floor(x0));
            x2 = x1+1;
            y1 = int(floor(y0));
            y2 = y1+1;
            dx = x0-floor(x0);
            dy = y0-floor(y0);

            b[p*dim_x*dim_y + i*dim_y + j] = 0.0;

            if ((x1 >= 0) && (dim_x - 1 >= x1) && (y1 >= 0) && (dim_y - 1 >= y1))
                b[p*dim_x*dim_y + i*dim_y + j] += (1.0-dx)*(1.0-dy)*a[p*dim_x*dim_y + x1*dim_y + y1];
            if ((x2 >= 0) && (dim_x - 1 >= x2) && (y1 >= 0) && (dim_y - 1 >= y1))
                b[p*dim_x*dim_y + i*dim_y + j] += dx*(1.0-dy)*a[p*dim_x*dim_y + x2*dim_y + y1];
            if ((x1 >= 0) && (dim_x - 1 >= x1) && (y2 >= 0) && (dim_y - 1 >= y2))
                b[p*dim_x*dim_y + i*dim_y + j] += (1.0-dx)*dy*a[p*dim_x*dim_y + x1*dim_y + y2];
            if ((x2 >= 0) && (dim_x - 1 >= x2) && (y2 >= 0) && (dim_y - 1 >= y2))
                b[p*dim_x*dim_y + i*dim_y + j] += dx*dy*a[p*dim_x*dim_y + x2*dim_y + y2];
        }}
    }
	return(0);
}
__declspec(dllexport) int Scale(int dim, int dim_x, int dim_y, double* a, double* center, double factor, double* b)
{
	double x0, y0, dx, dy;
	int x1, x2, y1, y2;

    for (int p=0; p<dim; p++)
    {
        /* visit all pixels of the output image and assign their value */
        for (int i = 0; i < dim_x; i++) {
        for (int j = 0; j < dim_y; j++) {

	        x0 = ((double)i-center[0]) / factor + center[0];
            y0 = ((double)j-center[1]) / factor + center[1];

            x1 = int(floor(x0));
            x2 = x1+1;
            y1 = int(floor(y0));
            y2 = y1+1;
            dx = x0-floor(x0);
            dy = y0-floor(y0);

            b[p*dim_x*dim_y + i*dim_y + j] = 0.0;

            if ((x1 >= 0) && (dim_x - 1 >= x1) && (y1 >= 0) && (dim_y - 1 >= y1))
                b[p*dim_x*dim_y + i*dim_y + j] += (1.0-dx)*(1.0-dy)*a[p*dim_x*dim_y + x1*dim_y + y1];
            if ((x2 >= 0) && (dim_x - 1 >= x2) && (y1 >= 0) && (dim_y - 1 >= y1))
                b[p*dim_x*dim_y + i*dim_y + j] += dx*(1.0-dy)*a[p*dim_x*dim_y + x2*dim_y + y1];
            if ((x1 >= 0) && (dim_x - 1 >= x1) && (y2 >= 0) && (dim_y - 1 >= y2))
                b[p*dim_x*dim_y + i*dim_y + j] += (1.0-dx)*dy*a[p*dim_x*dim_y + x1*dim_y + y2];
            if ((x2 >= 0) && (dim_x - 1 >= x2) && (y2 >= 0) && (dim_y - 1 >= y2))
                b[p*dim_x*dim_y + i*dim_y + j] += dx*dy*a[p*dim_x*dim_y + x2*dim_y + y2];
        }}
    }
	return(0);
}
}
