# -*- coding: iso_8859_1 -*-

import numpy as np
from ctypes import *

#----------------------------------------------------------------------------------------
#def add_sigma_clipping(images, sigma_coef, iterations):
#
#--- purpose ---
#to combine a list of images with the sigma clipping method
#the method works pixel by pixel: for one pixel, it calculates the mean and std_deviation values on all images, then the pixels deviating from the mean by sigma_coef * std_deviation are rejected
#the process is repeated iterations times, then the mean pixel value is kept for the final image
#the method is not recommended for a low number (<15) of images. For a higher number of images it is very efficient
#
#--- input parameters ---
#images = 3D numpy array containing image data
#		images[i][j][k] contains the pixel of the ith image, on jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#		for an Audine KAF0400, j=0 to 511 and k=0 to 767
#sigma_coef = the pixel that deviates from the mean value by more than sigma_coef * std_deviation is rejected to compute value for the final image
#iterations = the process of calculating mean, std deviation and testing to reject additional pixel values is repeated iterations times
#
#--- output parameters ---
#result = 2D numpy array containing data of averaged image, with only the 2 image dimensions (image-out[j][k])
#----------------------------------------------------------------------------------------

def add_sigma_clipping(images, sigma_coef, iterations):

	const = np.ones(images.shape)
	filter = const

	for k in range(np.int(iterations)):

		mean = np.sum(images*filter, axis=0) / np.sum(filter, axis=0)
		std = np.sqrt( np.sum((images-mean*const)**2., axis=0) / np.sum(filter, axis=0) ) 
		filter = np.array(np.abs(images-mean*const) <= np.tile(sigma_coef*std,(images.shape[0],1,1)), dtype=np.float32)

	return np.sum(images*filter, axis=0) / np.sum(filter, axis=0)

#----------------------------------------------------------------------------------------
#def add_mean(images):
#
#--- purpose ---
#computes the mean image from a list of images
#
#--- input parameters ---
#images = 3D numpy array containing image data
#		images[i][j][k] contains the pixel of the ith image, on jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#		for an Audine KAF0400, j=0 to 511 and k=0 to 767
#
#--- output parameters ---
#result = 2D numpy array containing data of the mean image, with only the 2 image dimensions (image-out[j][k])
#----------------------------------------------------------------------------------------

def add_mean(images):
	return np.mean(images, axis=0)
	
#----------------------------------------------------------------------------------------
#def add_mean(images):
#
#--- purpose ---
#computes the median image from a list of images
#an efficient C++ algorithm has been programmed for median computation, and compiled as a DLL. The DLL is called from Python with the module c_types
#
#--- input parameters ---
#images = 3D numpy array containing image data
#		images[i][j][k] contains the pixel of the ith image, on jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#		for an Audine KAF0400, j=0 to 511 and k=0 to 767
#
#--- output parameters ---
#result = 2D numpy array containing data of the median image, with only the 2 image dimensions (image-out[j][k])
#----------------------------------------------------------------------------------------

def add_median(images):

	dim, dim_x, dim_y = images.shape

	imag_in = c_double * (dim*dim_x*dim_y) 
	y=imag_in()
	y[:] = images.flatten()[:]

	imag_out = c_double * (dim_x*dim_y) 
	z=imag_out()

	cdll.median.Median(dim,dim_x,dim_y,byref(y),byref(z))

	result = []
	result[:] = z[:]
	return np.reshape(result,(dim_x,dim_y))
