# -*- coding: iso_8859_1 -*-

import datetime, time
import numpy as np

#------------------------------------------------------------------------------------------
#	possible headers
#------------------------------------------------------------------------------------------

h_obs = ['OBSERVER','Jerome CARON','Observer name']
h_ori = ['ORIGIN','Uiterstegracht, Leiden, Netherlands','Origin place of FITS file']
h_lat = ['SITELAT','52:09:26.6 North','Latitude of data acquisition']
h_long = ['SITELONG','04:29:51.9 East','Longitude of data acquisition']

h_tel_1 = ['TELESCOP','Vixen VISAC VC200L + focal reducer','Data acquisition telescope']
h_foc_1 = ['FOCAL',1280.,'Focal length (mm)']
h_tel_2 = ['TELESCOP','Vixen VISAC VC200L','Data acquisition telescope']
h_foc_2 = ['FOCAL',1800.,'Focal length (mm)']
h_dia = ['DIAMETER',200.,'Diameter (mm)']

h_ccd = ['INSTRUME','Audine Kaf0402-ME','Data acquisition instrument']
h_binX = ['BINX',1,'X binning']
h_binY = ['BINY',1,'Y binning']

#------------------------------------------------------------------------------------------

def create_headers(object = '', telescope = '', exposure = -1., date_obs=''):

	if telescope == 'visac_short_f':
		headerlist = [h_obs, h_ori, h_lat, h_long, h_ccd, h_tel_1, h_foc_1, h_dia, h_binX, h_binY]

	elif telescope == 'visac_long_f':	
		headerlist = [h_obs, h_ori, h_lat, h_long, h_ccd, h_tel_2, h_foc_2, h_dia, h_binX, h_binY]
	
	else:
		headerlist = [h_obs, h_ori, h_lat, h_long, h_ccd, h_binX, h_binY]

	if (object != ''):
		#adding object
		headerlist = headerlist + [['OBJECT', object, '']]

	if (exposure != -1.):
		#adding mean exposure time
		headerlist = headerlist + [['EXPOSURE',exposure,'Exposure (s)']]
	
	if (date_obs != ''):
		#adding DATE-OBS
		headerlist = headerlist + [['DATE-OBS', date_obs, 'Date of sky observation']]

	#adding date of file writing
	now = datetime.datetime.now()
	date = now.strftime("%Y-%m-%dT%H:%M:%S")
	headerlist = headerlist + [['DATE',str(date),'Date of file writing']]
	
	return headerlist

def julian_day(dateobs):
	time_format = "%Y-%m-%dT%H:%M:%S"
	result = []
	for dd in dateobs:
		dd2 = dd.split('.')[0]
		dd3 = datetime.datetime.fromtimestamp(time.mktime(time.strptime(dd2, time_format)))

		year = dd3.year
		month = dd3.month
		
		day = dd3.day + dd3.hour/24. + dd3.minute/(24.*60.) + dd3.second/(24.*3600.)
		
		if len(dd.split('.'))>1:
			if len(dd.split('.')[1])>0:
				day = day + float('.'+dd.split('.')[1])/(24.*3600.)

		if month<=2:
			month = month+12
			year = year-1
		
		a = np.trunc(year/100.)
		b = 2. - a + np.trunc(a/4.)
			
		result.append(np.trunc(365.25*year)+np.trunc(30.6001*(month+1.))+day+1720994.5+b)
	return result