# -*- coding: iso_8859_1 -*-

import numpy as np
import load, process, transform
import displaying

def aperture_photometry(Folder, Imagelist, xy, sizes, psf_fit_area):

	print
	print '---------------------------------------------------------------------'
	print '      script for aperture photometry'
	print '---------------------------------------------------------------------'
	
	#loop on images
	mag_total = []
	for i in range(len(Imagelist)):

		#loads relevant image
		print 'Photometry on :',Imagelist[i]
		image = load.getdata(Folder, [Imagelist[i]], text_display='silent')

		#loop on stars / asteroids
		mag_stars = []
		for j in range(len(xy[i])):

			#best integer coordinates for object center
			x_best = np.round(xy[i][j][0])
			y_best = np.round(xy[i][j][1])

			#------ re-centering accurately the star ------
			if 	(x_best-psf_fit_area<0)or(x_best+psf_fit_area+1>image.shape[0])or(y_best-psf_fit_area<0)or(y_best+psf_fit_area+1>image.shape[1]):
				print "new PSF fit not possible on image ", i+1, ": object", j+1, "too close from image border"
				new_fit = False
			else:
				pp = process.fit_gauss_elliptical([x_best-psf_fit_area, y_best-psf_fit_area],image[x_best-psf_fit_area:x_best+psf_fit_area+1, y_best-psf_fit_area:y_best+psf_fit_area+1])
				if np.abs(x_best-pp[3])+np.abs(y_best-pp[4])>4.:
					print "new PSF fit not possible on image ", i+1, ": too large shift for object", j+1
					new_fit = False
				else:
					new_fit = True
			#----------------------------------------------
					
			#loop on apertures / PSF area sizes for photometry
			mag_methods = []
			for k in range(len(sizes)):		

				#test has additional margin due to difference between x_best and pp[3], y_best and pp[4]
				if 	(x_best-sizes[k][0]-2<0)or(x_best+sizes[k][0]+3>image.shape[0])or(y_best-sizes[k][0]-2<0)or(y_best+sizes[k][0]+3>image.shape[1]):
					print "aperture photometry not possible on image", i+1, ": object", j+1, "too close from image border"
					mag_methods.append(0.)
				else:
					#selecting subimage
					xnot = max(0,x_best-sizes[k][2]-2)
					xrange = min(image.shape[0],x_best+sizes[k][2]+3)
					ynot = max(0,y_best-sizes[k][2]-2)
					yrange = min(image.shape[1],y_best+sizes[k][2]+3)

					subimage = image[xnot:xrange,ynot:yrange]
					#------ re-centering accurately the star ------
					if new_fit:
						subimage = transform.translate_bilinear(subimage,(x_best-pp[3],y_best-pp[4]))
					else:
						subimage = transform.translate_bilinear(subimage,(x_best-xy[i][j][0],y_best-xy[i][j][1]))
					#----------------------------------------------
					
					subimage = subimage[2:-2,2:-2]

					#aperture photometry
					grid = np.indices(subimage.shape)
					radius = (grid[0]-x_best+xnot+2)**2+(grid[1]-y_best+ynot+2)**2

					cond1 = radius<=sizes[k][0]**2
					cond2 = (radius>=sizes[k][1]**2)*(radius<=sizes[k][2]**2)

					center = np.extract(cond1,subimage)
					crown = np.extract(cond2,subimage)

					floor = np.ma.median(np.array(crown,dtype=np.float))
					intensity = np.sum(np.array(center,dtype=np.float)-floor)
					if intensity <= 0.:
						print "negative intensity on image", i+1, "for object", j+1
						mag_methods.append(999.)
					else:
						mag_methods.append(-2.5*np.log10(intensity))

			mag_stars.append(mag_methods)
		mag_total.append(mag_stars)

	return mag_total
	