# -*- coding: iso_8859_1 -*-

import astro
import os, datetime

#------------------------------------------------------------------------------------
#	input parameters
#------------------------------------------------------------------------------------

#--- images ---
#Folder = u"C:\\Documents and Settings\\Jrme\\Bureau\\06mar2011\\processing python\\Chryseis\\"
#Folder = u"C:\\Documents and Settings\\jerome caron\\Desktop\\Perso astro\\06mar2011\\processing python\\Chryseis\\"

Folder = u"C:\\Documents and Settings\\Jrme\\Bureau\\15mar2011\\processing\\Chryseis\\"
Imagelist = ["Chryseis-calibrated-" + str(i+1) + ".fit" for i in range(341)]


#--- output file ---
outputfile = "magnitudes.txt"

#parameters to find image shifts
threshold = 0.1		#for star detection
psf_fit_area = 10		#for PSF fitting

#radii for aperture photometry (pixels)
#sizes = [[4.5,8,18],[4.5,8,22],[5,10,20],[5,10,24],[5.5,10,20],[5.5,10,24],[6,10,20],[6,10,24],[6.5,10,20],[6.5,10,24],[7,12,24],[7,12,28]]
#sizes = [[2.5,8,18],[2.5,8,22],[3,8,18],[3,8,22],[3.5,8,18],[3.5,8,22],[4,8,18],[4,8,22],[7.5,14,26],[7.5,14,32],[8,16,26],[8,16,32],[8.5,16,30],[8.5,16,36],[9,16,30],[9,16,36],[9.5,16,30],[9.5,16,36]]
#sizes = [[2.5,8,18],[3,8,18],[3.5,8,18],[4,8,18],[4.5,8,18],[5,10,20],[5.5,10,20],[6,10,20],[6.5,10,20],[7,12,24],[7.5,14,26],[8,16,26],[8.5,16,30],[9,16,30],[9.5,16,30]]
sizes = [[3,8,22],[3.5,8,22],[4,8,22],[4.5,8,22],[5,10,24],[5.5,10,24],[6,10,24],[6.5,10,24],[7,12,28]]
#sizes = [[5,10,24]]

#------------------------------------------------------------------------------------
#	calling photometry function
#------------------------------------------------------------------------------------

#------ identification of target and reference stars ------
names, xy = astro.select_stars(Folder, Imagelist, threshold, psf_fit_area, images_aligned='no')

#------ aperture photometry ------
mag_total = astro.aperture_photometry(Folder, Imagelist, xy, sizes, psf_fit_area)

#------ date information ------
dateobs = astro.julian_day(astro.getheadervalues(Folder, Imagelist, 'DATE-OBS'))

#------------------------------------------------------------------------------------
#	saving data
#------------------------------------------------------------------------------------

if os.path.exists(Folder+outputfile):
	os.remove(Folder+outputfile)

f = open(Folder+outputfile, 'w')

#--- file header ---
f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
f.write(str(len(Imagelist))+' images processed\n')
f.write('first image = '+(Folder).encode('iso_8859_1')+Imagelist[0]+'\n\n')

#--- stars X,Y coordinates ---
f.write('object coordinates\n')
for j in range(len(xy[0])):
	f.write('X '+names[j]+'\t'+'Y '+names[j]+'\t')
f.write('\n')
for i in range(len(Imagelist)):
	for j in range(len(xy[i])):
		f.write(str(xy[i][j][0])+'\t'+str(xy[i][j][1])+'\t')
	f.write('\n')
f.write('\n')

#--- measures magnitudes ---
for k in range(len(sizes)):
	if len(sizes[k])==1:
		f.write('magnitudes from PSF fitting over area +/- '+str(sizes[k][0])+' pixels wide\n')
	elif len(sizes[k])==3:
		f.write('magnitudes from apertures: '+str(sizes[k][0])+' - '+str(sizes[k][1])+' - '+str(sizes[k][2])+'\n')

	f.write('Julian day\t')
	for j in range(len(xy[i])):
		f.write(names[j]+'\t')
	f.write('\n')
	for i in range(len(Imagelist)):
		f.write(str(dateobs[i])+'\t')
		for j in range(len(xy[i])):
			f.write(str(mag_total[i][j][k])+'\t')
		f.write('\n')
	f.write('\n')

f.close()

raw_input("ok")