# -*- coding: iso_8859_1 -*-

import astro
import numpy as np
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------------
#	information on source FITS data
#------------------------------------------------------------------------------------

#--- input frames ---
source_Folder = u"C:\\Documents and Settings\\jerome caron\\Desktop\\Perso astro\\06mar2011\\sources\\"
Darklist = ["Dark\\Dark-" + str(i+1) + ".fits" for i in range(20)]
Flatlist = ["flat-final-02mar2011.fit"]
Image = "Chryseis\\Chryseis-"
Imagelist = [Image + str(i+1) + ".fits" for i in range(184)] + [Image + str(i+1) + ".fits" for i in range(186,368)]

#--- output frames ---
processing_Folder = u"C:\\Documents and Settings\\jerome caron\\Desktop\\Perso astro\\06mar2011\\processing python\\"
Saving = "Chryseis\\Chryseis-aligned-"
Savinglist = [Saving + str(i+1) + ".fit" for i in range(184)] + [Saving + str(i+1) + ".fit" for i in range(186,368)]

#--- additional information (for FITS headers) ---
Tel = 'visac_short_f'
Obj = 'Chryseis - calibrated and aligned'

#parameters to find image shifts
threshold = 0.05		#for star detection
psf_fit_area = 12		#for PSF fitting

#------------------------------------------------------------------------------------
#	preparing calibration frames
#------------------------------------------------------------------------------------

darks = astro.getdata(source_Folder, Darklist)
dark_final = astro.add_sigma_clipping(darks, 3., 1)

flat = astro.getdata(processing_Folder, Flatlist)
flat = flat / astro.get_median(flat)

#------------------------------------------------------------------------------------
#	reference image
#------------------------------------------------------------------------------------

Exp = astro.getheadervalues(source_Folder, Imagelist, 'EXPOSURE')
Dateobs = astro.getheadervalues(source_Folder, Imagelist, 'DATE-OBS')

#--- Processing and saving ---
image0 = astro.getdata(source_Folder, [Imagelist[0]])
image0_aligned = (image0 - dark_final) / flat
print 'dark and flat calibration ... done'
headerlist = astro.create_headers(object=Obj, telescope=Tel, exposure=Exp[0], date_obs = Dateobs[0])
astro.save_fits(image0_aligned, headerlist, processing_Folder+Savinglist[0])
print Savinglist[0] + ' saved'

#--- Choosing zone to exclude for shift calculation ---
astro.display(image0_aligned)

print
print 'Specify boundaries of zone to be excluded to calculate shift'
print 'if moving object (asteroid), only exclude a few pixels giving its position on 1st image'
xmin = np.float32(raw_input('x boundaries (vertical): xmin = '))
xmax = np.float32(raw_input('x boundaries (vertical): xmax = '))
ymin = np.float32(raw_input('y boundaries (horizontal): ymin = '))
ymax = np.float32(raw_input('y boundaries (horizontal): ymax = '))

plt.close()

#------------------------------------------------------------------------------------
#	images calibration, co-registration and saving
#------------------------------------------------------------------------------------

for i in range(1,len(Imagelist)):
	#loading
	images = astro.getdata(source_Folder, [Imagelist[i]])

	#calibration
	images_calib = (images - dark_final) / flat
	print 'dark and flat calibration ... done'

	#calculate shifts
	dx,dy = astro.find_shift_stars_matching(image0_aligned, images_calib, threshold, psf_fit_area, excluded = [xmin, xmax, ymin, ymax])
	print 'shifts (vert, hor) wrt reference image:', dx, dy

	#shift image
	images_aligned = astro.translate_bilinear(images_calib, (-dx,-dy))

	#save image
	headerlist = astro.create_headers(object=Obj, telescope=Tel, exposure=Exp[i], date_obs = Dateobs[i])
	astro.save_fits(images_aligned, headerlist, processing_Folder+Savinglist[i])
	print Savinglist[i] + ' saved'

raw_input("Press ENTER to close")

