# -*- coding: iso_8859_1 -*-

import astro
import numpy as np
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------------
#	information on source FITS images
#------------------------------------------------------------------------------------

Folder = u"C:\\Documents and Settings\\jerome caron\\Desktop\\Perso astro\\21fev2011\\sources\\"

#--- frames ---
Darklist = ["dark\\dark-" + str(i+1) + ".fit" for i in range(20)]
Flat = "flat-final.fit"
Imagelist = ["Chryseis\\Chryseis-" + str(i+1) + ".fit" for i in range(41)]

#--- additional information (for FITS headers) ---
Tel = 'visac_short_f'
Obj = 'NGC891'

#parameters to find image shifts
threshold = 0.01		#for star detection
psf_fit_area = 12		#for PSF fitting

#------------------------------------------------------------------------------------
#	loading all images
#------------------------------------------------------------------------------------

darks = astro.getdata(Folder, Darklist)
flat = astro.getdata(Folder, Flat)
images = astro.getdata(Folder, Imagelist)

#------------------------------------------------------------------------------------
#	radiometric calibration
#------------------------------------------------------------------------------------

print
print '--- radiometric calibration ---'
dark_final = astro.add_sigma_clipping(darks, 3., 1)
print 'dark-final ... generated'
images_calib = (images - np.tile(dark_final,(len(Imagelist),1,1))) * astro.get_median(flat) / np.tile(flat,(len(Imagelist),1,1))
print 'dark and flat calibration ... done'

#------------------------------------------------------------------------------------
#	Choosing zone to exclude for shift calculation
#------------------------------------------------------------------------------------

astro.display(images_calib[0,:,:])

print
print 'Specify boundaries of zone to be excluded to calculate shift'
print 'if moving object (asteroid), only exclude a few pixels giving its position on 1st image'
xmin = np.float32(raw_input('x boundaries (vertical): xmin = '))
xmax = np.float32(raw_input('x boundaries (vertical): xmax = '))
ymin = np.float32(raw_input('y boundaries (horizontal): ymin = '))
ymax = np.float32(raw_input('y boundaries (horizontal): ymax = '))

plt.close()

#------------------------------------------------------------------------------------
#	image co-registration and co-addition
#------------------------------------------------------------------------------------

images_aligned = np.zeros(images_calib.shape)
images_aligned[0,:,:] = images_calib[0,:,:]

print 
print "--- computing shifts (vertical, horizontal) ---"
print 0.,0.
for i in range(1,len(Imagelist)):
	dx,dy = astro.find_shift_stars_matching(images_calib[0,:,:], images_calib[i,:,:], threshold, psf_fit_area, excluded = [xmin, xmax, ymin, ymax])
	print dx, dy
	images_aligned[i,:,:] = astro.translate_bilinear(images_calib[i,:,:], (-dx,-dy))

#image_final = astro.add_sigma_clipping(images_aligned, 3., 1)
image_final = astro.add_mean(images_aligned)

print
print"--- statistics of final image ---"
print "min, mean, max"
print np.min(image_final.flatten()), np.mean(image_final.flatten()), np.max(image_final.flatten())
astro.display(image_final)

#------------------------------------------------------------------------------------
#	saving final image.fit
#------------------------------------------------------------------------------------

Exp = astro.getheadervalues(Folder, Imagelist, 'EXPOSURE')
Dateobs = astro.getheadervalues(source_Folder, Imagelist, 'DATE-OBS')

headerlist = astro.create_headers(object=Obj, telescope=Tel, exposure=np.mean(Exp), date_obs = Dateobs[0])
astro.save_fits(image_final, headerlist, Folder+'image-final.fit')

raw_input("Press ENTER to close")

